@@ -87,6 +87,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
     UIView *_sleepTimerContainer;
     UIDatePicker *_sleepTimeDatePicker;
+    NSTimer *_sleepCountDownTimer;
 
     NSInteger _mediaDuration;
 }
@@ -398,7 +399,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
             }
         }
         _sleepTimeDatePicker.datePickerMode = UIDatePickerModeCountDownTimer;
-        _sleepTimeDatePicker.minuteInterval = 5;
+        _sleepTimeDatePicker.minuteInterval = 1;
         _sleepTimeDatePicker.minimumDate = [NSDate date];
         _sleepTimeDatePicker.countDownDuration = 1200.;
         [_sleepTimeDatePicker addTarget:self action:@selector(sleepTimerAction:) forControlEvents:UIControlEventValueChanged];
@@ -799,6 +800,34 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     [_playPauseButtonLandscape setImage:playPauseImage forState:UIControlStateNormal];
 }
 
+- (void)updateTimeDisplayButton
+{
+    VLCMediaPlayer *mediaPlayer = self.playbackController.mediaPlayer;
+    UIButton *timeDisplayButton = self.timeNavigationTitleView.timeDisplayButton;
+    if (_displayRemainingTime)
+        [timeDisplayButton setTitle:[[mediaPlayer remainingTime] stringValue] forState:UIControlStateNormal];
+    else
+        [timeDisplayButton setTitle:[[mediaPlayer time] stringValue] forState:UIControlStateNormal];
+    [self.timeNavigationTitleView setNeedsLayout];
+}
+
+- (void)updateSleepTimerButton
+{
+    NSMutableString *title = [NSMutableString stringWithString:NSLocalizedString(@"BUTTON_SLEEP_TIMER", nil)];
+    VLCPlaybackController *vpc = [VLCPlaybackController sharedInstance];
+    if (vpc.sleepTimer != nil && vpc.sleepTimer.valid) {
+        int remainSeconds = (int)[vpc.sleepTimer.fireDate timeIntervalSinceNow];
+        int hour = remainSeconds / 3600;
+        int minute = (remainSeconds - hour * 3600) / 60;
+        int second = remainSeconds % 60;
+        [title appendFormat:@"  %02d:%02d:%02d", hour, minute, second];
+    } else {
+        [_sleepCountDownTimer invalidate];
+    }
+
+    [_sleepTimerButton setTitle:title forState:UIControlStateNormal];
+}
+
 #pragma mark - playback controller delegation
 
 - (VLCPlaybackController *)playbackController
@@ -816,11 +845,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
         self.timeNavigationTitleView.positionSlider.value = [mediaPlayer position];
     }
 
-    if (_displayRemainingTime)
-        [self.timeNavigationTitleView.timeDisplayButton setTitle:[[mediaPlayer remainingTime] stringValue] forState:UIControlStateNormal];
-    else
-        [self.timeNavigationTitleView.timeDisplayButton setTitle:[[mediaPlayer time] stringValue] forState:UIControlStateNormal];
-    [self.timeNavigationTitleView setNeedsLayout];
+    [self updateTimeDisplayButton];
 }
 
 - (void)prepareForMediaPlayback:(VLCPlaybackController *)controller
@@ -976,6 +1001,7 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
     LOCKCHECK;
 
     _displayRemainingTime = !_displayRemainingTime;
+    [self updateTimeDisplayButton];
 
     [self _resetIdleTimer];
 }
@@ -1004,6 +1030,14 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 {
     VLCPlaybackController *vpc = [VLCPlaybackController sharedInstance];
     [vpc scheduleSleepTimerWithInterval:_sleepTimeDatePicker.countDownDuration];
+
+    if (_sleepCountDownTimer == nil || _sleepCountDownTimer.valid == NO) {
+        _sleepCountDownTimer = [NSTimer scheduledTimerWithTimeInterval:1
+                                                                   target:self
+                                                                 selector:@selector(updateSleepTimerButton)
+                                                                 userInfo:nil
+                                                                  repeats:YES];
+    }
 }
 
 - (void)moreActions:(id)sender
@@ -1107,6 +1141,9 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
             }
         }
 
+        if (SYSTEM_RUNS_IOS7_OR_LATER)
+            _sleepTimerContainer.hidden = YES;
+
         self.videoFilterView.hidden = _videoFiltersHidden = YES;
     } else {
         _trackSelectorContainer.hidden = YES;
