@@ -55,6 +55,7 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     LXReorderableCollectionViewFlowLayout *_reorderLayout;
     BOOL _inFolder;
     UILongPressGestureRecognizer *_longPressGestureRecognizer;
+    UITapGestureRecognizer *_tapTwiceGestureRecognizer;
 
     NSMutableArray *_searchData;
     UISearchBar *_searchBar;
@@ -74,6 +75,11 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
 
 @implementation VLCPlaylistViewController
 
+- (void)dealloc
+{
+    [_tapTwiceGestureRecognizer removeTarget:self action:NULL];
+}
+
 + (void)initialize
 {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
@@ -219,9 +225,9 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     _searchBar.delegate = self;
     _searchBar.hidden = YES;
 
-    UITapGestureRecognizer *tapTwiceGesture = [[UITapGestureRecognizer alloc] initWithTarget:self  action:@selector(tapTwiceGestureAction:)];
-    [tapTwiceGesture setNumberOfTapsRequired:2];
-    [self.navigationController.navigationBar addGestureRecognizer:tapTwiceGesture];
+    _tapTwiceGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self  action:@selector(tapTwiceGestureAction:)];
+    [_tapTwiceGestureRecognizer setNumberOfTapsRequired:2];
+    [self.navigationController.navigationBar addGestureRecognizer:_tapTwiceGestureRecognizer];
 
     _searchData = [[NSMutableArray alloc] init];
 }
@@ -625,6 +631,8 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
         MLFile* object = _foundMedia[fromIndexPath.item];
         [_foundMedia removeObjectAtIndex:fromIndexPath.item];
         [_foundMedia insertObject:object atIndex:toIndexPath.item];
+        if (![object isKindOfClass:[MLFile class]])
+            return;
         object.folderTrackNumber = @(toIndexPath.item - 1);
         object = [_foundMedia objectAtIndex:fromIndexPath.item];
         object.folderTrackNumber = @(fromIndexPath.item - 1);
@@ -814,7 +822,10 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
             [cell showMetadata:NO];
     }
 
-    NSManagedObject *selectedObject = _foundMedia[indexPath.row];
+    NSManagedObject *selectedObject;
+    @synchronized(self) {
+        selectedObject = _foundMedia[indexPath.row];
+    }
     [self openMediaObject:selectedObject];
 }
 
@@ -829,7 +840,10 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
 
 - (void)collectionView:(UICollectionView *)collectionView removeItemFromFolderAtIndexPathIfNeeded:(NSIndexPath *)indexPath
 {
-    id mediaObject = _foundMedia[indexPath.item];
+    id mediaObject;
+    @synchronized(self) {
+        mediaObject = _foundMedia[indexPath.item];
+    }
     if (![mediaObject isKindOfClass:[MLFile class]])
         return;
 
@@ -859,8 +873,12 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
 
 - (void)collectionView:(UICollectionView *)collectionView requestToMoveItemAtIndexPath:(NSIndexPath *)itemPath intoFolderAtIndexPath:(NSIndexPath *)folderPath
 {
-    id folderPathItem = _foundMedia[folderPath.item];
-    id itemPathItem = _foundMedia[itemPath.item];
+    id folderPathItem;
+    id itemPathItem;
+    @synchronized(self) {
+        folderPathItem = _foundMedia[folderPath.item];
+        itemPathItem = _foundMedia[itemPath.item];
+    }
 
     BOOL validFileTypeAtFolderPath = ([folderPathItem isKindOfClass:[MLFile class]] || [folderPathItem isKindOfClass:[MLLabel class]]) && [itemPathItem isKindOfClass:[MLFile class]];
 
@@ -940,10 +958,15 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     for (NSInteger i = _indexPaths.count - 1; i >=0; i--) {
         NSIndexPath *path = _indexPaths[i];
         id mediaObject;
-        if (!_usingTableViewToShowData)
-            mediaObject = _foundMedia[path.item];
-        else
-            mediaObject = _foundMedia[path.row];
+        if (!_usingTableViewToShowData) {
+            @synchronized(self) {
+                mediaObject = _foundMedia[path.item];
+            }
+        } else {
+            @synchronized(self) {
+                mediaObject = _foundMedia[path.row];
+            }
+        }
         if ([mediaObject isKindOfClass:[MLLabel class]])
             [_indexPaths removeObject:path];
     }
@@ -952,7 +975,9 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
         NSArray *folder = [MLLabel allLabels];
         //if we already have folders display them
         if ([folder count] > 0) {
-            _foundMedia = [NSMutableArray arrayWithArray:folder];
+            @synchronized(self) {
+                _foundMedia = [NSMutableArray arrayWithArray:folder];
+            }
             self.title = NSLocalizedString(@"SELECT_FOLDER", nil);
             _previousLibraryMode = _libraryMode;
             _libraryMode = VLCLibraryModeCreateFolder;
@@ -975,7 +1000,10 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
 
     for (NSInteger i = [_indexPaths count] - 1; i >= 0; i--) {
         NSIndexPath *path = _indexPaths[i];
-        id item = _foundMedia[_usingTableViewToShowData ? path.row : path.item];
+        id item;
+        @synchronized(self) {
+            item = _foundMedia[_usingTableViewToShowData ? path.row : path.item];
+        }
 
         if ([item isKindOfClass:[MLFile class]]) {
             MLFile *file = (MLFile *)item;
@@ -1015,10 +1043,16 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     }
 
     if (_folderObject != nil) {
-        NSUInteger folderIndex = [_foundMedia indexOfObject:_folderObject];
+        id mediaObject;
+        NSUInteger folderIndex;
+        @synchronized(self) {
+            folderIndex = [_foundMedia indexOfObject:_folderObject];
+            mediaObject = _foundMedia[folderIndex];
+        }
+
         //item got dragged onto item
-        if ([_foundMedia[folderIndex] isKindOfClass:[MLFile class]]) {
-            MLFile *file = _foundMedia[folderIndex];
+        if ([mediaObject isKindOfClass:[MLFile class]]) {
+            MLFile *file = (MLFile *)mediaObject;
             MLLabel *label = [[MLMediaLibrary sharedMediaLibrary] createObjectForEntity:@"Label"];
             label.name = folderName;
 
@@ -1044,27 +1078,34 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
             itemFile.folderTrackNumber = @([label files].count - 1);
         } else {
             //item got dragged onto folder or items should be added to folder
-            MLLabel *label = _foundMedia[folderIndex];
+            MLLabel *label;
+            @synchronized(self) {
+                label = _foundMedia[folderIndex];
+            }
             [_indexPaths sortUsingSelector:@selector(compare:)];
 
-            for (NSInteger i = [_indexPaths count] - 1; i >= 0; i--) {
-                NSIndexPath *path = _indexPaths[i];
-                if (_libraryMode != VLCLibraryModeCreateFolder && ![_foundMedia[path.row] isKindOfClass:[MLFile class]])
-                    continue;
-                if (_libraryMode == VLCLibraryModeCreateFolder)
-                    [self updateViewContents];
+            @synchronized(self) {
+                NSUInteger count = _foundMedia.count;
+                for (NSInteger i = [_indexPaths count] - 1; i >= 0; i--) {
+                    NSIndexPath *path = _indexPaths[i];
+                    if (path.row >= count)
+                        continue;
+                    if (_libraryMode != VLCLibraryModeCreateFolder && ![_foundMedia[path.row] isKindOfClass:[MLFile class]])
+                        continue;
+                    if (_libraryMode == VLCLibraryModeCreateFolder)
+                        [self updateViewContents];
 
-                id item = _foundMedia[path.row];
+                    id item = _foundMedia[path.row];
 
-                if (![item isKindOfClass:[MLFile class]])
-                    continue;
+                    if (![item isKindOfClass:[MLFile class]])
+                        continue;
 
-                MLFile *file = (MLFile *)item;
-                file.labels = [NSSet setWithObjects:label, nil];
-                @synchronized(self) {
+                    MLFile *file = (MLFile *)item;
+                    file.labels = [NSSet setWithObjects:label, nil];
                     [_foundMedia removeObjectAtIndex:path.row];
+
+                    file.folderTrackNumber = @([label files].count - 1);
                 }
-                file.folderTrackNumber = @([label files].count - 1);
             }
         }
         _folderObject = nil;
@@ -1137,6 +1178,8 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
         [visibleCells enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
             VLCPlaylistCollectionViewCell *aCell = (VLCPlaylistCollectionViewCell*)obj;
 
+            // always clear selection when enter edit mode
+            aCell.selected = NO;
             [aCell setEditing:editing animated:animated];
         }];
         self.collectionView.allowsMultipleSelection = editing;
